package	jp.co.sds_corp.transfer.source;

import javax.jms.JMSException;

import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.msg.IMessage;
import jp.co.sds_corp.util.msg.IMessageAdapter;
import jp.co.sds_corp.util.msg.IReceiveListener;
import jp.co.sds_corp.util.msg.MessageAdapterFactory;

/**
* データをJMSで受信します。
各要素の詳細については、{@link MessageAdapterFactory}を参照してください。
<PRE>
&lt;Source name="JMS" class="jp.co.sds_corp.transfer.source.JMSSource"&gt;
	&lt;Type&gt;jmsTopic&lt;/Type&gt;
	&lt;InitialContext&gt;weblogic.jndi.WLInitialContextFactory&lt;/InitialContext&gt;
	&lt;ConnectionFactory&gt;jms/topic/ConnectionFactory&lt;/ConnectionFactory&gt;
	&lt;URL&gt;http://localhost:8080&lt;/URL&gt;
	&lt;User&gt;user&lt;/User&gt;
	&lt;Password&gt;passwd&lt;/Password&gt;
	&lt;Receive&gt;jms/topic/result&lt;/Receive&gt;
&lt;/Source&gt;
</PRE>
*/
public class JMSSource extends AbstractSource implements IReceiveListener
{
	private static final long serialVersionUID = 1L;
	static CmnLogger logger = CmnLogger.getInstance();
	IMessageAdapter	adapter;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		try {
			MessageAdapterFactory factory = new MessageAdapterFactory();
			adapter = factory.connect( config );

			adapter.createReceiver( config.get( "Receive" ), null, false );
			adapter.addReceiveListener( this );
		} catch ( JMSException je ) {
			Exception le = je.getLinkedException();
			if ( le != null ) {
				logger.error( this, "linked exception:", le );
			}
			throw( je );
		}
	}

	/**
	* このサービスを開始します。
	*/
	public void start() {
		try {
			adapter.start();
		} catch ( JMSException je ) {
			Exception le = je.getLinkedException();
			if ( le != null ) {
				logger.error( this, "linked exception:", le );
			}
			logger.error( this, "start error", je );
		} catch ( Exception e ) {
			logger.error( this, "start error", e );
		}
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		try {
			adapter.stop();
		} catch ( JMSException je ) {
			Exception le = je.getLinkedException();
			if ( le != null ) {
				logger.error( this, "linked exception:", le );
			}
			logger.error( this, "stop error", je );
		} catch ( Exception e ) {
			logger.error( this, "stop error", e );
		}
	}

	/**
	* メッセージアダプタを返します。
	*/
	public IMessageAdapter getAdapter() {
		return adapter;
	}

	/**
	* JMSメッセージを受信した時に呼び出されます。
	* @param	mAdapter	受信したメッセージアダプタ
	* @param	cmnMessage	受信したメッセージ
	*/
	public void onMessage( IMessageAdapter mAdapter, IMessage cmnMessage )
	{
		try {
			ReceiveEvent e = (ReceiveEvent )cmnMessage.getObject( "Event" );
			fireReceive( e.getData() );
		} catch ( Exception ex ) {
			logger.error( this, "message receive error", ex );
		}
	}
}
